from typing import Any, IO

class JSONEncoder(object):
    def __init__(self, *args, **kwargs): ...
    def encode(self, o: Any): ...
    def default(self, o: Any): ...
    def iterencode(self, o: Any, _one_shot: bool): ...

class JSONEncoderForHTML(JSONEncoder): ...
